from datetime import datetime
import random
import os
from datetime import timedelta

def get_random_name_datetime():
    date_now = datetime.now()
    return "{}{}".format(date_now.strftime("%Y%m%d%H%M%S_"),random.randint(10000, 99999))

def delete_files_by_modification_hours(folder,hours_limit:int=8):
    now = datetime.now()
    time_limit = now - timedelta(hours=int(hours_limit))

    for file in os.listdir(folder):
        file_path = os.path.join(folder, file)
        if os.path.isfile(file_path):
            modification_time = os.path.getmtime(file_path)
            modification_datetime = datetime.fromtimestamp(modification_time)
            if modification_datetime < time_limit:
                os.remove(file_path)



from enum import StrEnum

class ContentType(StrEnum):
   """Lowercase characters must be used"""
   IMAGE_BASE64      = "image_base64"
   IMAGE_URL         = "image_url"
   PDF_BASE64        = "pdf_base64"
   PDF_URL           = "pdf_url"
   PLAIN_TEXT_BASE64 = "plain_text_base64"
   PLAIN_TEXT_URL    = "plain_text_url"
   TEXT              = "text"
   
   @classmethod
   def to_dict(self) -> str:
       return {item.name: item.value for item in ContentType}

   @classmethod
   def get_type_list(self) -> str:
       return [item.value for item in ContentType]

class ContentRole(StrEnum):
   """
   system:
      Messages that establish the assistant's behavior. These messages typically define the rules and tone the assistant should follow during the conversation.
      Consistency: Ensures the assistant follows a consistent behavior throughout the conversation.
      Personalization: Allows customization of the interaction according to the specific needs of the user or application.
      Context: Sets the initial context, which is crucial for the assistant to understand the purpose of the conversation and act accordingly.

   user:
      Messages sent by the user. They represent the user's queries, commands, or interactions with the assistant.

   assistant:
      Responses generated by the assistant. These messages contain the responses that the language model generates based on the previous messages in the conversation.
   """
   SYSTEM = "system"
   USER = "user"
   ASSISTANT = "assistant"

   def find_by_key(value)->str|None:
      for variable in ContentRole:
         if str(variable.name).lower() == str(value).lower():
               return variable.value   



from logyca_ai.utils.schemes.input.conversations import (
    AssistantMessage,
    Content,
    ImageFileMessage,
    PdfFileMessage,
    PlainTextFileMessage,
    UserMessage,
)
from logyca_ai.assets_for_examples.file_or_documents.image_base64 import image_base64_sample
from logyca_ai.assets_for_examples.file_or_documents.pdf_base64 import pdf_base64_sample
from logyca_ai.utils.constants.content import ContentType
from logyca_ai.utils.constants.image import ImageResolution
from logyca_ai.utils.helpers.content_loaders import load_text_from_url
import base64

def encode_str_base64(message: str) -> str:
    """
    Encodes a given message into Base64.

    :param message: The message to encode.
    :return: The encoded message in Base64.
    """
    try:
        message_bytes = message.encode('utf-8')
        base64_bytes = base64.b64encode(message_bytes)
        return base64_bytes.decode('utf-8')
    except Exception as e:
        raise ValueError(f"Could not decode base64 string {e}")

def get_content_simple_sample()->Content:
    return Content(
        system="""
                    Voy a definirte tu personalidad, contexto y proposito.
                    Actua como un experto en venta de frutas.
                    Se muy positivo.
                    Trata a las personas de usted, nunca tutees sin importar como te escriban.
                """.strip(),
        messages=[
            UserMessage(user="Dime 5 frutas amarillas"),
            AssistantMessage(assistant="""
                    ¡Claro! Aquí te van 5 frutas amarillas:

                    1. Plátano
                    2. Piña
                    3. Mango
                    4. Melón
                    5. Papaya
                """
            ),
            UserMessage(user="Dame los nombres en ingles."),
        ]
        )

def get_content_image_sample(image_sample_base64:bool=False)->Content:
    image_resolution=str(ImageResolution.AUTO)
    if image_sample_base64:
        base64_content_or_url=image_base64_sample
        image_format="png"
        type_message=ContentType.IMAGE_BASE64
    else:
        base64_content_or_url="https://raw.githubusercontent.com/logyca/python-libraries/main/logyca-ai/logyca_ai/assets_for_examples/file_or_documents/image.png"
        image_format=ContentType.IMAGE_URL
        type_message=ContentType.IMAGE_URL
    return Content(
        system="""
                Actua como una maquina lectora de imagenes.
                Devuelve la información sin lenguaje natural, sólo responde lo que se está solicitando.
                El dispositivo que va a interactuar contigo es una api, y necesita la información sin markdown u otros caracteres especiales.
                """.strip(),
        messages=[
            UserMessage(
                user="Extrae el texto que recibas en la imagen y devuelvelo en formato json.",
                type=type_message,
                additional_content=ImageFileMessage(
                    base64_content_or_url=base64_content_or_url,
                    image_format=image_format,
                    image_resolution=image_resolution,
                ).to_dict()
            )
        ]
    )

def get_content_pdf_sample(pdf_sample_base64:bool=False)->Content:
    if pdf_sample_base64:
        base64_content_or_url=pdf_base64_sample
        pdf_format="pdf"
        type_message=ContentType.PDF_BASE64
    else:
        base64_content_or_url="https://raw.githubusercontent.com/logyca/python-libraries/main/logyca-ai/logyca_ai/assets_for_examples/file_or_documents/pdf.pdf"
        pdf_format=ContentType.PDF_URL
        type_message=ContentType.PDF_URL
    return Content(
        system="""
                No uses lenguaje natural para la respuesta.
                Dame la información que puedas extraer de la imagen en formato JSON.
                Solo devuelve la información, no formatees con caracteres adicionales la respuesta.
                """.strip(),
        messages=[
            UserMessage(
                user="Dame los siguientes datos: Expediente, radicación, Fecha, Numero de registro, Vigencia.",
                type=type_message,
                additional_content=PdfFileMessage(
                    base64_content_or_url=base64_content_or_url,
                    pdf_format=pdf_format,
                ).to_dict()
            )
        ]
    )

def get_content_plain_text_sample(file_sample_base64:bool=False)->Content:
    url = "https://raw.githubusercontent.com/logyca/python-libraries/main/logyca-ai/logyca_ai/assets_for_examples/file_or_documents/plain_text.csv"
    if file_sample_base64:
        data=load_text_from_url(url)
        base64_content_or_url=encode_str_base64(data)
        file_format="csv"
        type_message=ContentType.PLAIN_TEXT_BASE64
    else:
        base64_content_or_url=url
        file_format=ContentType.PLAIN_TEXT_URL
        type_message=ContentType.PLAIN_TEXT_URL
    return Content(
        system="""
                No uses lenguaje natural para la respuesta.
                Dame la información que puedas extraer en formato JSON.
                Solo devuelve la información, no formatees con caracteres adicionales la respuesta.
                Te voy a enviar un texto que representa información en formato csv.
                """.strip(),
        messages=[
            UserMessage(
                user="""
                Dame los siguientes datos de la primera fila del documento: Expediente, radicación, Fecha, Numero de registro, Vigencia.
                A partir de la fila 2 del documento, suma los valores de la columna Valores_A.
                A partir de la fila 2 del documento, Suma los valores de la columna Valores_B.
                """.strip(),
                type=type_message,
                additional_content=PlainTextFileMessage(
                    base64_content_or_url=base64_content_or_url,
                    file_format=file_format,
                ).to_dict()
            )
        ]
    )








from logyca_ai.assets_for_examples.conversation_samples import (
    get_content_image_sample,
    get_content_pdf_sample,
    get_content_plain_text_sample,
    get_content_simple_sample,
    )
from logyca_ai.utils.constants.content import ContentRole, ContentType
from logyca_ai.utils.constants.image import ImageResolution
from logyca_ai.utils.helpers.azure_openai_chatgpt import AzureOpenAIChatGPT
from logyca_ai.utils.schemes.input.conversations import (
    AssistantMessage,
    Content,
    ImageFileMessage,
    PdfFileMessage,
    PlainTextFileMessage,
    UserMessage,
    )
from logyca_ai.utils.helpers.general_utils import get_random_name_datetime, delete_files_by_modification_hours
from logyca_ai.utils.helpers.text_extraction import extract_text_from_pdf_file
from logyca_ai.utils.helpers.content_loaders import (
    get_base64_from_file,
    load_text_from_url,
    save_base64_to_file,
    save_file_from_url,
)


from enum import StrEnum
from PIL import Image # Pillow
import fitz  # PyMuPDF
import os
import pytesseract

class OCREngine(StrEnum):
    WINDOWS_PATH_01     = "C:\\Program Files\\Tesseract-OCR\\tesseract.exe"
    WINDOWS_PATH_02     = "C:\\Program Files (x86)\\Tesseract-OCR\\tesseract.exe"
    LINUX_PATH_01       = "/usr/bin/tesseract"
    LINUX_PATH_02       = "/usr/local/bin/tesseract"
    
    @classmethod
    def get_binary_path(cls):
        for location in cls:
            if os.path.exists(location):
                return str(location)
        return None

def extract_text_from_pdf_file(filename_full_path:str,advanced_image_recognition:bool=False,ocr_engine_path:str=None,output_temp_dir:str=None):
    """
    Extracts text from a PDF file.

    :param filename_full_path: Full path to the PDF file from which to extract text.
    :type filename_full_path: str
    :param advanced_image_recognition: Indicates whether to perform text recognition on images within the PDF.
                               If True, OCR techniques will be used to extract text from images.
    :type advanced_image_recognition: bool
    :param ocr_engine_path: Path to the OCR executable. If provided, this path will be used instead of the default.
    :type ocr_engine_path: str, optional
    :param output_temp_dir: Temporary directory for storing output files.
                            If not provided, a default tmp temporary directory in the application root folder will be used.
    :type output_temp_dir: str, optional

    :return: Extracted text from the PDF file.
    :rtype: str

    :raises FileNotFoundError: If the specified PDF file is not found.
    :raises ValueError: If the OCR path is invalid.

    :example:

    # Example usage
    ```python
    text = extract_text_from_pdf_file('/tmp/example.pdf', advanced_image_recognition=True, ocr_engine_path='/usr/local/bin/tesseract', output_temp_dir='/tmp/tesseract_output')
    ```

    """

    if advanced_image_recognition:
        if ocr_engine_path is None:
            pytesseract.pytesseract.tesseract_cmd=OCREngine.get_binary_path()
        else:
            pytesseract.pytesseract.tesseract_cmd=ocr_engine_path

    doc = fitz.open(filename_full_path)

    text = ""
    if output_temp_dir is None:
        output_temp_dir=os.path.abspath(os.path.join(os.path.dirname(__file__),"tmp"))
    if not os.path.exists(output_temp_dir):
        os.makedirs(output_temp_dir)

    for page_num in range(len(doc)):
        page = doc.load_page(page_num)

        text += page.get_text()
        if advanced_image_recognition:
            image_list = page.get_images(full=True)
            for img_index, img in enumerate(image_list):
                xref = img[0]
                base_image = doc.extract_image(xref)
                image_bytes = base_image["image"]

                image_path = os.path.join(output_temp_dir, f"image_{page_num+1}_{img_index+1}.png")
                with open(image_path, "wb") as img_file:
                    img_file.write(image_bytes)

                image = Image.open(image_path)
                ocr_text = pytesseract.image_to_string(image)
                text += ocr_text

                os.remove(image_path)

    doc.close()
    return text


#!/usr/bin/env python
from setuptools import setup, find_packages

COMPANY_NAME="LOGYCA"
PACKAGE_NAME = "logyca-ai"
VERSION = "0.2.0a6"

install_requires = ["pydantic>=2.5","openai>=1.37.1","logyca>=0.1.17","PyMuPDF>=1.23.2","pillow>=9.3.0","pytesseract>=0.3.9"]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=f'An integration package created by the company {COMPANY_NAME} to interact with ChatGPT and analyze documents, files and other functionality of the OpenAI library.',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    license='MIT License',
    author='Jaime Andres Cardona Carrillo',
    author_email='jacardona@outlook.com',
    url='https://github.com/logyca/python-libraries/tree/main/logyca-ai',
    keywords="artificial-intelligence, machine-learning, deep-learning, chatgpt, nlp, language-models, openai, transformers, neural-networks, ai-tools, mlops, data-science, python, data-analysis, automation",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python",
        "Topic :: Database",
        "Topic :: Database :: Front-Ends",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development",
        "Typing :: Typed",
    ],
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=install_requires,
)
